# -*- coding: utf-8 -*-
import datetime

from odoo import Command, SUPERUSER_ID
from odoo.addons.saas_worker.compat import route, http, request
from odoo.addons.website.controllers.main import Website
from odoo.addons.website_sale.controllers.main import WebsiteSale

from werkzeug.exceptions import Forbidden

import logging
_logger = logging.getLogger(__name__)


class OdooPricing(Website):

    @route(['/pricing'], auth='public', website=True, sitemap=True, readonly=True)
    def odoo_enterprise_pricing(self, **kwargs):
        return request.redirect('/pricing-plan')

    @route(['/pricing-plan'], auth='public', website=True, sitemap=True, readonly=True)
    def odoo_enterprise_pricing_plan(self, **kwargs):
        prices = request.website.sudo().get_pricing()
        return request.render("openerp_enterprise.pricing-plan", dict(prices=prices))

    @route(['/pricing-configurator'], auth='public', website=True, sitemap=True, readonly=True)
    def pricing_configurator(self, **kwargs):
        is_custom = kwargs.get('plan') != 'standard'
        prices = request.website.sudo().get_pricing().copy()

        # integrating partners
        learning_partner_id = request.env.ref('openerp_portal.res_partner_grade_learning').id
        country = self._buy_get_country()
        partner_proxy = request.env['res.partner'].sudo()
        base_partner_domain = [('is_company', '=', True), ('grade_id', '!=', False),
                               ('website_published', '=', True),
                               '|', ('grade_id.website_published', '=', True), ('grade_id', '=', learning_partner_id),
                               ('country_id', '=', country.id)]
        # Retrieve all the partners for this country
        partners = partner_proxy.search(base_partner_domain, order="grade_id asc,name")
        # Extract the grades
        grades = partners.mapped('grade_id')
        # Regroups the partners by their grades
        partner_by_grades = {}
        for g in grades:
            partner_by_grades[g.id] = [{'id': p.id, 'name': p.name} for p in partners if p.grade_id.id == g.id]
        # List the grades in a tuple with their id & name.
        partner_grades = [(g.id, g.name) for g in grades]

        data = {
            'prices': prices,
            'is_custom': is_custom,
            'partner_grades': partner_grades,
            'partner_by_grades': partner_by_grades,
            'im_partner': request.env.user.partner_id.commercial_partner_id in partners and request.env.user.partner_id.commercial_partner_id,
        }
        if is_custom:
            data['odoo_sh'] = data['prices'].pop('odoo_sh')
        data['packs'] = data['prices'].pop('packs')
        return request.render("openerp_enterprise.pricing-configurator", data)

    @route('/pricing-packs', auth='public', website=True, sitemap=True, readonly=True)
    def odoo_website_pack(self, **kwargs):
        prices = request.website.sudo().get_pricing(with_packs=True)

        params = {
            'prices': prices,
            'packs': prices['packs'],
            'currency_record': prices['pl'].currency_id,
        }

        return request.render("openerp_enterprise.pricing-packs", params)

    @http.route('/pricing/checkout', auth='public', methods=['POST'], website=True)
    def odoo_pricing_checkout(self, hosting, num_users, price_by, **kwargs):
        # pricelist_id = int(kwargs.get('pl'))

        # if the user is trying to fraud by paying an on premise with the monthly plan
        if price_by == 'monthly' and hosting == 'on_premise':
            return Forbidden()

        if int(num_users) < 1:
            num_users = 1

        # we create the sale order
        so = request.website.odoo_get_order(
            periodicity=price_by,
            num_users=int(num_users),
            hosting=hosting,
            force_create=True,
            **kwargs
        )
        if not so.origin:
            so.origin = 'SaaS Subscription from pricing page'

        return request.redirect('/shop/checkout')

    @http.route('/pricing/quote', auth='public', methods=['POST'], type='json', website=True)
    def odoo_pricing_quote(self, hosting, num_users, price_by, **kwargs):
        new_partner_values = {
            'name': kwargs.get('name'),
            'phone': kwargs.get('phone'),
            'email': kwargs.get('email'),
            'street': kwargs.get('street'),
            'zip': kwargs.get('zip'),
            'city': kwargs.get('city'),
            'country_id': int(kwargs.get('country_id')),
            'company_name': kwargs.get('company_name'),
            'vat': kwargs.get('vat'),
        }
        if kwargs.get('state_id'):
            new_partner_values['state_id'] = int(kwargs['state_id'])

        new_partner = request.env['res.partner'].sudo().create(new_partner_values)
        sale_order = request.env['sale.order'].sudo().create(dict(partner_id=new_partner.id))

        # odoo_pricing_checkout -> odoo_get_order -> sale_get_order will automatically change the
        # customer with the current connected user (if connected). So we create a new cart to avoid
        # to poluate the current one and avoid to add others products on the quote (e.g. from appstore).
        # And at the end, we restore the old cart to avoid that when the partner see the quote, he
        # replace the customer by himself.
        old_cart_id = request.session.get('sale_order_id')
        request.session['sale_order_id'] = sale_order.id

        self.odoo_pricing_checkout(hosting, num_users, price_by, **kwargs)
        sale_order.partner_id = new_partner.id
        self_pricing = request.env.ref('openerp_enterprise.crm_tag_self_quote_pricing').id
        sale_order.tag_ids = [Command.link(self_pricing)]

        # If no saleman, set employee as saleman
        if not sale_order.user_id and not request.env.user.share:
            sale_order.user_id = request.env.user.id
        if sale_order.user_id and sale_order.company_id != sale_order.user_id.company_id:
            sale_order.company_id = sale_order.user_id.company_id

        # restore my old personal cart
        request.session['sale_order_id'] = old_cart_id

        # send quote by email
        if kwargs.get('mode') == 'SEND':
            try:
                send_to = request.env['res.partner'].sudo().find_or_create(kwargs.get('email_to'))
                sale_order.message_subscribe(send_to.ids)
            except ValueError:
                return '/pricing?invalid_email=1'
            email_ctx = sale_order.action_quotation_send().get('context', {})
            email_ctx['mail_notify_force_send'] = True
            template = request.env["mail.template"].browse(email_ctx.get('default_template_id'))
            sale_order.with_user(SUPERUSER_ID).with_context(**email_ctx).message_post_with_source(
                template,
                email_layout_xmlid="mail.mail_notification_layout_with_responsible_signature",
                subtype_xmlid='mail.mt_comment',
                partner_ids=send_to.ids
            )
            return sale_order.get_portal_url() + '&pricing_sent=1'
        else:  # print mode
            return sale_order.get_portal_url(report_type='pdf', download=True)

    def _buy_get_country(self):
        """
        Return the country for the self buying process.

        Take the country from GeoIP if user is not logged; otherwise use partner's country. If the
        user is an Odoo employee, allow spoofing locations.
        """
        country_code = request.geoip.country_code or 'BE'
        country = request.env['res.country'].search([('code', '=', country_code)])
        if request.website.partner_id != request.env.user.partner_id:
            partner_id = request.env.user.partner_id
            # allow localization spoofing for employees only
            country = country if request.env.user.has_group('base.group_user') else partner_id.country_id
        return country


class WebsiteSale(WebsiteSale):
    @http.route()
    def cart_update_json(self, product_id, line_id=None, add_qty=None, set_qty=None, display=True, **kw):
        order = request.website.sale_get_order(force_create=1)
        if order.state != 'draft':
            request.website.sale_reset()
            return {}

        # we fetch the product
        product = request.env['product.product'].sudo().browse(product_id)

        # if the product is either an Odoo app or a user discount
        user_mapped_to_discounts = {
            request.env.ref('openerp_enterprise.product_odoo_user').id: request.env.ref('openerp_enterprise.product_odoo_user_discount').id,
            request.env.ref('openerp_enterprise.product_odoo_user_standard').id: request.env.ref('openerp_enterprise.product_odoo_user_standard_discount').id,
            request.env.ref('openerp_enterprise.product_odoo_user_custom').id: request.env.ref('openerp_enterprise.product_odoo_user_custom_discount').id,
        }

        elearning_partnership = request.env.ref('openerp_enterprise.odoo_learning_platform_access')

        # if the user wants to remove an app, users or eLearning partnership, then we remove all the order line that are related to odoo checkout
        if set_qty == 0 and (product == elearning_partnership or product.is_odoo_user or product.is_odoo_app):
            order_lines_to_remove = order.order_line.filtered(
                lambda x: x.product_id == elearning_partnership or x.product_id.is_odoo_app or x.product_id.is_odoo_user
            )
            order_lines_to_remove.unlink()

            # to avoid sending mail with the Odoo subscription template
            order.write({
                'origin': '',
                'sale_order_template_id': None,
                'referrer_id': None,
                'client_order_ref': '',
            })

            return self.render_cart_line(order)

        if product.is_odoo_app or product.id in user_mapped_to_discounts.values():
            # should not happen's since apps and discount are not sellable line.
            return self.render_cart_line(order)

        # if the customer changed his user count, then we do the same for the user discounts
        if product.id in user_mapped_to_discounts.keys():
            discount = user_mapped_to_discounts[product.id]
            order._cart_update(product_id=discount, add_qty=add_qty, set_qty=set_qty, **kw)

        return super(WebsiteSale, self).cart_update_json(product_id, line_id, add_qty, set_qty, display, **kw)


    def render_cart_line(self, order):
        value = dict()
        value['cart_quantity'] = order.cart_quantity
        value['cart_ready'] = order._is_cart_ready()
        from_currency = order.company_id.currency_id
        to_currency = order.pricelist_id.currency_id
        value['website_sale.cart_lines'] = request.env['ir.ui.view']._render_template("website_sale.cart_lines", {
            'website_sale_order': order,
            'compute_currency': lambda price: from_currency.compute(price, to_currency),
            'suggested_products': order._cart_accessories(),
            'date': datetime.date.today(),
        })
        value['website_sale.total'] = request.env['ir.ui.view']._render_template(
            "website_sale.total", {
                'website_sale_order': order,
            }
        )
        return value

    @http.route()
    def cart_update(self, product_id, add_qty=1, set_qty=0, **kw):
        product = request.env['product.product'].browse(int(product_id))
        # if the user tries to add/remove an odoo app/user (for example if he tries to fraud the user discount) then we dismiss the request
        if product and (product.is_odoo_app or product.is_odoo_user):
            return request.redirect("/shop/cart")

        return super(WebsiteSale, self).cart_update(product_id, add_qty, set_qty, **kw)

    def values_postprocess(self, order, mode, values, errors, error_msg):
        new_values, errors, error_msg = super(WebsiteSale, self).values_postprocess(order, mode, values, errors, error_msg)
        if order.user_id:
            new_values['user_id'] = order.user_id.id
        return new_values, errors, error_msg

    def _get_mandatory_fields_billing(self, country_id=False):
        return super()._get_mandatory_fields_billing(country_id=country_id) + ["phone"]
